==> enterprisemodules-ora_config-3.22.2.tar.gz: enterprisemodules-ora_config-3.22.2/lib/puppet/type/ora_rman_config.rb <==
# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/oracle/access'
require 'puppet_x/enterprisemodules/oracle/commands'
require 'puppet_x/enterprisemodules/oracle/information'
require 'puppet_x/enterprisemodules/oracle/default_sid'
require 'puppet_x/enterprisemodules/oracle/title_parser'
require 'puppet_x/enterprisemodules/extend_hash'

# @nodoc
Puppet::Type.newtype(:ora_rman_config) do
  include EasyType
  include EasyType::ArrayProperty
  include PuppetX::EnterpriseModules::Oracle::Access
  include PuppetX::EnterpriseModules::Oracle::Commands
  include PuppetX::EnterpriseModules::Oracle::DefaultSid
  extend PuppetX::EnterpriseModules::Oracle::TitleParser
  include PuppetX::EnterpriseModules::ExtendHash

  desc <<-DESC
    This type allows you to manage the RMAN configurration in an Oracle database

    ora_rman_config { 'SID':
      archivelog_deletion_policy                         => 'BACKED UP 1 TIMES TO DISK',
      controlfile_autobackup                             => 'ON',
      controlfile_autobackup_format_for_device_type_disk => 'SID_%F',
      device_type_disk                                   => 'PARALLELISM 2 BACKUP TYPE TO COMPRESSED BACKUPSET',
      maxsetsize                                         => '1024G',
    }
  DESC

  # ensurable

  SUPPORTED_PROPS = [
    'ARCHIVELOG BACKUP COPIES',
    'ARCHIVELOG DELETION POLICY',
    'BACKUP OPTIMIZATION',
    'CHANNEL',
    'COMPRESSION ALGORITHM',
    'CONTROLFILE AUTOBACKUP FORMAT',
    'CONTROLFILE AUTOBACKUP',
    'DATAFILE BACKUP COPIES',
    'DB_UNIQUE_NAME',
    'DEFAULT DEVICE TYPE',
    'DEVICE',
    'ENCRYPTION ALGORITHM',
    'ENCRYPTION FOR DATABASE',
    'MAXSETSIZE',
    'RETENTION POLICY',
    'RMAN OUTPUT',
    'SNAPSHOT CONTROLFILE NAME'
  ].freeze

  TO_PROPS = [
    'ARCHIVELOG BACKUP COPIES',
    'CONTROLFILE AUTOBACKUP FORMAT',
    'DATAFILE BACKUP COPIES',
    'RETENTION POLICY',
    'DEFAULT DEVICE TYPE',
    'MAXSETSIZE',
    'RMAN OUTPUT',
    'ARCHIVELOG DELETION POLICY',
    'SNAPSHOT CONTROLFILE NAME'
  ].freeze

  SPACE_PROPS = [
    'BACKUP OPTIMIZATION',
    'CHANNEL',
    'COMPRESSION ALGORITHM',
    'CONTROLFILE AUTOBACKUP',
    'DEVICE',
    'ENCRYPTION ALGORITHM',
    'ENCRYPTION FOR DATABASE'
  ].freeze

  QUOTED_PROPERTIES = [
    'DB_UNIQUE_NAME',
    'DEFAULT DEVICE TYPE',
    'ENCRYPTION ALGORITHM',
    'SNAPSHOT CONTROLFILE NAME'
  ].freeze

  to_get_raw_resources do
    show_all_raw = rman_on_all_database_sids('show all')
    show_all_raw.collect do |show_all_db|
      show_all_config = show_all_db['OUTPUT'].each_line.grep(/^CONFIGURE/)
      show_all_config = show_all_config.map do |line|
        line.gsub(/'SBT_TAPE'/, 'SBT').
          gsub(/SBT_TAPE/, 'SBT').
          gsub(/CONFIGURE /, '').
          gsub(/; # default\n/, ' (default)').
          gsub(/;\n/, '')
      end
      entry = {}
      entry['SID'] = show_all_db['SID']
      available_props = SUPPORTED_PROPS
      show_all_config.collect do |line|
        prop_values = self.set_prop_values(line, available_props)
        current_prop = prop_values[0]
        sep = prop_values[1]
        next if current_prop.nil?

        case current_prop
        when 'ARCHIVELOG BACKUP COPIES'
          # Get all ARCHIVELOG BACKUP COPIES properties
          copies = show_all_config.grep(/ARCHIVELOG BACKUP COPIES/)
          copies_hsh = copies.reduce({}) do |memo, copy|
            copies_array = copy.match(/ARCHIVELOG BACKUP COPIES FOR DEVICE TYPE (SBT|DISK) TO (.*)/)
            memo.merge!(copies_array[1] => copies_array[2])
          end
          entry['ARCHIVELOG_BACKUP_COPIES'] = copies_hsh
        when 'CHANNEL'
          # Get all CHANNEL properties
          channels = show_all_config.grep(/CHANNEL/)
          ch_hash = channels.sort.reduce({}) do |memo, ch|
            ch_array = ch.match(/CHANNEL( | \d+ )DEVICE TYPE (SBT|DISK) (\w*.*)/)
            ch_name = ch_array[1].strip.empty? ? 'DEFAULT' : ch_array[1].strip
            memo.merge!({ ch_array[2] => { ch_name => ch_array[3] } }) { |_key, memo_value, merge_value| memo_value.merge(merge_value) }
          end
          entry['CHANNEL'] = ch_hash
        when 'CONTROLFILE AUTOBACKUP FORMAT'
          # Get all CONTROLFILE AUTOBACKUP FORMAT properties
          caf = show_all_config.grep(/CONTROLFILE AUTOBACKUP FORMAT/)
          caf_hsh = caf.reduce({}) do |memo, form|
            caf_array = form.match(/CONTROLFILE AUTOBACKUP FORMAT FOR DEVICE TYPE (SBT|DISK) TO (.*)/)
            memo.merge!(caf_array[1] => caf_array[2].tr('\'', ''))
          end
          entry['CONTROLFILE_AUTOBACKUP_FORMAT'] = caf_hsh
        when 'DATAFILE BACKUP COPIES'
          # Get all DATAFILE BACKUP COPIES properties
          copies = show_all_config.grep(/DATAFILE BACKUP COPIES/)
          copies_hsh = copies.reduce({}) do |memo, copy|
            copies_array = copy.match(/DATAFILE BACKUP COPIES FOR DEVICE TYPE (SBT|DISK) TO (.*)/)
            memo.merge!(copies_array[1] => copies_array[2])
          end
          entry['DATAFILE_BACKUP_COPIES'] = copies_hsh
        when 'DB_UNIQUE_NAME'
          # There might be more than one property for DB_UNIQUE_NAME
          unique_names = show_all_config.select { |conf| conf.include?('DB_UNIQUE_NAME') }
          unique_hsh = unique_names.reduce({}) do |memo, copy|
            unique_array = copy.match(/.*'(.*)' CONNECT IDENTIFIER.*'(.*)'/)
            memo.merge!(unique_array[1] => unique_array[2])
          end
          entry['DB_UNIQUE_NAME'] = unique_hsh
        when 'DEVICE'
          # Get all DEVICE properties
          device = show_all_config.grep(/^DEVICE/)
          device_hsh = device.reduce({}) do |memo, dev|
            device_array = dev.match(/DEVICE TYPE (SBT|DISK) (.*)/)
            memo.merge!(device_array[1] => device_array[2])
          end
          entry['DEVICE'] = device_hsh
        else
          entry[current_prop.tr(' ', '_')] = QUOTED_PROPERTIES.include?(current_prop) ? line.gsub("#{current_prop}#{sep}", '').tr('\'', '').strip : line.gsub("#{current_prop}#{sep}", '').strip
        end

        # Remove current property from available properties because we have some overlapping properties
        # and there might be undefined properties
        available_props -= [current_prop]
      end
      entry
    end
  end

  on_create do
    nil
  end

  on_modify do
    # Delegate to properties
    nil
  end

  on_destroy do
    nil
  end

  map_title_to_sid(:sid)

  parameter :name
  parameter :sid
  parameter :target_conn
  parameter :catalog_conn

  property :archivelog_backup_copies
  property :archivelog_deletion_policy
  property :backup_optimization
  property :channel
  property :compression_algorithm
  property :controlfile_autobackup
  property :controlfile_autobackup_format
  property :datafile_backup_copies
  property :db_unique_name
  property :default_device_type
  property :device
  property :encryption_algorithm
  property :encryption_for_database
  property :maxsetsize
  property :retention_policy
  property :rman_output
  property :snapshot_controlfile_name

  def self.set_prop_values(line, available_props)
    out_array = []
    available_props.each do |prop|
      next unless line.match?(/^#{prop}/)

      out_array << prop
      if TO_PROPS.include?(prop)
        out_array <<  ' TO '
      elsif SPACE_PROPS.include?(prop)
        out_array <<  ' '
      end
    end
    out_array
  end
end

==> enterprisemodules-ora_config-3.23.4.tar.gz: enterprisemodules-ora_config-3.23.4/lib/puppet/type/ora_rman_config.rb <==
# frozen_string_literal: true

#
# See the file "LICENSE" for the full license governing this code.
#
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../.."))
$LOAD_PATH.unshift(File.absolute_path("#{__dir__}/../../../../easy_type/lib"))
require 'easy_type'
require 'puppet_x/enterprisemodules/oracle/access'
require 'puppet_x/enterprisemodules/oracle/commands'
require 'puppet_x/enterprisemodules/oracle/information'
require 'puppet_x/enterprisemodules/oracle/default_sid'
require 'puppet_x/enterprisemodules/oracle/title_parser'
require 'puppet_x/enterprisemodules/extend_hash'

# @nodoc
Puppet::Type.newtype(:ora_rman_config) do
  include EasyType
  include EasyType::ArrayProperty
  include PuppetX::EnterpriseModules::Oracle::Access
  include PuppetX::EnterpriseModules::Oracle::Commands
  include PuppetX::EnterpriseModules::Oracle::DefaultSid
  extend PuppetX::EnterpriseModules::Oracle::TitleParser
  include PuppetX::EnterpriseModules::ExtendHash

  desc <<-DESC
    This type allows you to manage the RMAN configurration in an Oracle database

    ora_rman_config { 'SID':
      archivelog_deletion_policy                         => 'BACKED UP 1 TIMES TO DISK',
      controlfile_autobackup                             => 'ON',
      controlfile_autobackup_format_for_device_type_disk => 'SID_%F',
      device_type_disk                                   => 'PARALLELISM 2 BACKUP TYPE TO COMPRESSED BACKUPSET',
      maxsetsize                                         => '1024G',
    }
  DESC

  # ensurable

  SUPPORTED_PROPS = [
    'ARCHIVELOG BACKUP COPIES',
    'ARCHIVELOG DELETION POLICY',
    'BACKUP OPTIMIZATION',
    'CHANNEL',
    'COMPRESSION ALGORITHM',
    'CONTROLFILE AUTOBACKUP FORMAT',
    'CONTROLFILE AUTOBACKUP',
    'DATAFILE BACKUP COPIES',
    'DB_UNIQUE_NAME',
    'DEFAULT DEVICE TYPE',
    'DEVICE',
    'ENCRYPTION ALGORITHM',
    'ENCRYPTION FOR DATABASE',
    'MAXSETSIZE',
    'RETENTION POLICY',
    'RMAN OUTPUT',
    'SNAPSHOT CONTROLFILE NAME'
  ].freeze

  TO_PROPS = [
    'ARCHIVELOG BACKUP COPIES',
    'CONTROLFILE AUTOBACKUP FORMAT',
    'DATAFILE BACKUP COPIES',
    'RETENTION POLICY',
    'DEFAULT DEVICE TYPE',
    'MAXSETSIZE',
    'RMAN OUTPUT',
    'ARCHIVELOG DELETION POLICY',
    'SNAPSHOT CONTROLFILE NAME'
  ].freeze

  SPACE_PROPS = [
    'BACKUP OPTIMIZATION',
    'CHANNEL',
    'COMPRESSION ALGORITHM',
    'CONTROLFILE AUTOBACKUP',
    'DEVICE',
    'ENCRYPTION ALGORITHM',
    'ENCRYPTION FOR DATABASE'
  ].freeze

  QUOTED_PROPERTIES = [
    'DB_UNIQUE_NAME',
    'DEFAULT DEVICE TYPE',
    'ENCRYPTION ALGORITHM',
    'SNAPSHOT CONTROLFILE NAME'
  ].freeze

  to_get_raw_resources do
    show_all_raw = rman_on_all_database_sids('show all')
    show_all_raw.collect do |show_all_db|
      show_all_config = show_all_db['OUTPUT'].each_line.grep(/^CONFIGURE/)
      show_all_config = show_all_config.map do |line|
        line.gsub("'SBT_TAPE'", 'SBT').
          gsub('SBT_TAPE', 'SBT').
          gsub('CONFIGURE ', '').
          gsub("; # default\n", ' (default)').
          gsub(";\n", '')
      end
      entry = {}
      entry['SID'] = show_all_db['SID']
      available_props = SUPPORTED_PROPS
      show_all_config.collect do |line|
        prop_values = self.set_prop_values(line, available_props)
        current_prop = prop_values[0]
        sep = prop_values[1]
        next if current_prop.nil?

        case current_prop
        when 'ARCHIVELOG BACKUP COPIES'
          # Get all ARCHIVELOG BACKUP COPIES properties
          copies = show_all_config.grep(/ARCHIVELOG BACKUP COPIES/)
          copies_hsh = copies.reduce({}) do |memo, copy|
            copies_array = copy.match(/ARCHIVELOG BACKUP COPIES FOR DEVICE TYPE (SBT|DISK) TO (.*)/)
            memo.merge!(copies_array[1] => copies_array[2])
          end
          entry['ARCHIVELOG_BACKUP_COPIES'] = copies_hsh
        when 'CHANNEL'
          # Get all CHANNEL properties
          channels = show_all_config.grep(/CHANNEL/)
          ch_hash = channels.sort.reduce({}) do |memo, ch|
            ch_array = ch.match(/CHANNEL( | \d+ )DEVICE TYPE (SBT|DISK) (\w*.*)/)
            ch_name = ch_array[1].strip.empty? ? 'DEFAULT' : ch_array[1].strip
            memo.merge!({ ch_array[2] => { ch_name => ch_array[3] } }) { |_key, memo_value, merge_value| memo_value.merge(merge_value) }
          end
          entry['CHANNEL'] = ch_hash
        when 'CONTROLFILE AUTOBACKUP FORMAT'
          # Get all CONTROLFILE AUTOBACKUP FORMAT properties
          caf = show_all_config.grep(/CONTROLFILE AUTOBACKUP FORMAT/)
          caf_hsh = caf.reduce({}) do |memo, form|
            caf_array = form.match(/CONTROLFILE AUTOBACKUP FORMAT FOR DEVICE TYPE (SBT|DISK) TO (.*)/)
            memo.merge!(caf_array[1] => caf_array[2].tr('\'', ''))
          end
          entry['CONTROLFILE_AUTOBACKUP_FORMAT'] = caf_hsh
        when 'DATAFILE BACKUP COPIES'
          # Get all DATAFILE BACKUP COPIES properties
          copies = show_all_config.grep(/DATAFILE BACKUP COPIES/)
          copies_hsh = copies.reduce({}) do |memo, copy|
            copies_array = copy.match(/DATAFILE BACKUP COPIES FOR DEVICE TYPE (SBT|DISK) TO (.*)/)
            memo.merge!(copies_array[1] => copies_array[2])
          end
          entry['DATAFILE_BACKUP_COPIES'] = copies_hsh
        when 'DB_UNIQUE_NAME'
          # There might be more than one property for DB_UNIQUE_NAME
          unique_names = show_all_config.select { |conf| conf.include?('DB_UNIQUE_NAME') }
          unique_hsh = unique_names.reduce({}) do |memo, copy|
            unique_array = copy.match(/.*'(.*)' CONNECT IDENTIFIER.*'(.*)'/)
            memo.merge!(unique_array[1] => unique_array[2])
          end
          entry['DB_UNIQUE_NAME'] = unique_hsh
        when 'DEVICE'
          # Get all DEVICE properties
          device = show_all_config.grep(/^DEVICE/)
          device_hsh = device.reduce({}) do |memo, dev|
            device_array = dev.match(/DEVICE TYPE (SBT|DISK) (.*)/)
            memo.merge!(device_array[1] => device_array[2])
          end
          entry['DEVICE'] = device_hsh
        else
          entry[current_prop.tr(' ', '_')] = QUOTED_PROPERTIES.include?(current_prop) ? line.gsub("#{current_prop}#{sep}", '').tr('\'', '').strip : line.gsub("#{current_prop}#{sep}", '').strip
        end

        # Remove current property from available properties because we have some overlapping properties
        # and there might be undefined properties
        available_props -= [current_prop]
      end
      entry
    end
  end

  on_create do
    nil
  end

  on_modify do
    # Delegate to properties
    nil
  end

  on_destroy do
    nil
  end

  map_title_to_sid(:sid)

  parameter :name
  parameter :sid
  parameter :target_conn
  parameter :catalog_conn

  property :archivelog_backup_copies
  property :archivelog_deletion_policy
  property :backup_optimization
  property :channel
  property :compression_algorithm
  property :controlfile_autobackup
  property :controlfile_autobackup_format
  property :datafile_backup_copies
  property :db_unique_name
  property :default_device_type
  property :device
  property :encryption_algorithm
  property :encryption_for_database
  property :maxsetsize
  property :retention_policy
  property :rman_output
  property :snapshot_controlfile_name

  def self.set_prop_values(line, available_props)
    out_array = []
    available_props.each do |prop|
      next unless line.match?(/^#{prop}/)

      out_array << prop
      if TO_PROPS.include?(prop)
        out_array <<  ' TO '
      elsif SPACE_PROPS.include?(prop)
        out_array <<  ' '
      end
    end
    out_array
  end
end
